const cli = require('chalk');
const { fs } = require('./../bootstrap/modules');
var execute = require('child-process-promise').exec;
var spawn = require('child-process-promise').spawn;

const echo = console.log;

class WorkSpace {
    constructor(command) {
        this.command = command;
        this.createNewWorkSpace();
    }

    async createNewWorkSpace() {
        echo(cli.yellow(`Creating ${this.command.appName} workspace...`));

        this.workspaceDirectory = `${ROOT}/${this.command.appName}`;

        if (fs.exists(this.workspaceDirectory)) {
            die(cli.redBright(`Workspace "${cli.cyan(this.workspaceDirectory)}" is not empty, please use another directory or application name.`));
        }

        fs.makeDirectory(this.workspaceDirectory);

        await execute(`npm init -y`, {
            cwd: this.workspaceDirectory,
        });

        try {
            echo(cli.cyan(`Installing dependencies...`));
            let result = await execute(`npm i falak`, {
                cwd: this.workspaceDirectory,
            });

            echo(result.stdout);

            echo(cli.green('Dependencies have been installed successfully'));
            
            echo('Configuring workspace...');
            // await execute(`flk fresh-install ${this.command.appName}`);
            var promise = spawn('flk', ['fresh-install', this.command.appName], {
                cwd: this.workspaceDirectory,
            });

            var childProcess = promise.childProcess;

            childProcess.stdout.on('data', data => {
                echo(data.toString());
            });

            childProcess.stderr.on('data', data => {
                die(cli.red(data.toString()));
            });

            childProcess.on('exit', () => {
                die();
            });
        } catch (e) {
            die(cli.red(e));
        }
    }
}

module.exports = command => new WorkSpace(command);