const { fs } = require('./../bootstrap/modules');

let htaccessFileContent = fs.get(FRAMEWORK_OUTPUTS_PATH + '/.htaccess');

let htaccessRulesPlaceholder = `
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteCond %{REQUEST_FILENAME} !-f
    RewriteRule ^appRule appName-locale.html [L] 
`;

function createHtaccessFile(command, apps) {
    let htacessRulesList = [],
        mainApp = '',
        appsList = '',
        APPS_LIST = [];
    for (let i = 0; i < apps.length; i++) {
        let appName = apps[i];

        let app = config.apps[appName],
            appPath = app.path.ltrim('/');

        let isBaseApp = i == apps.length - 1;
        if (! isBaseApp) {
            APPS_LIST.push(appName);
        }

        for (let localeCode of app.locales) {
            let appRule = localeCode == app.locale ? appPath : `${appPath}/${localeCode}`;

            if (isBaseApp && localeCode == app.locale) {
                mainApp = `${appName}-${localeCode}`;
            } else {
                if (isBaseApp && localeCode != app.locale) {
                    appRule = localeCode;
                    APPS_LIST.push(localeCode);
                }

                let htaccessAppsRules = htaccessRulesPlaceholder.replace('appRule', appRule)
                    .replace('appName', appName)
                    .replace('locale', localeCode);

                if (isBaseApp || ! isBaseApp && localeCode == app.locale) {
                    htacessRulesList.push(htaccessAppsRules);
                } else {
                    htacessRulesList.unshift(htaccessAppsRules);
                }
            }
        }
    }

    if (! Is.empty(APPS_LIST)) {
        appsList = `(${APPS_LIST.join('|')})|`;
    }

    let outputHtaccessFileContent = htaccessFileContent.replace('main', mainApp).replace('APPS_LIST', appsList).replace('#apps-rules', htacessRulesList.join(""));

    fs.put(ROOT + '/dist/.htaccess', outputHtaccessFileContent);
}

async function startProducing(command) {
    if (!fs.exists(MAIN_CONFIG_PATH)) {
        die(cli.white(`Framework is not configured, please run ${cli.green(process.env.CLI_NAME + ' fresh-install')} first.`));
    }

    let config = fs.getJson(MAIN_CONFIG_PATH);
    // remove the old dist
    fs.remove(ROOT + '/dist');
    // create again
    fs.makeDirectory(ROOT + '/dist');

    let apps = ! Is.empty(command.args.apps) ? command.args.apps : Object.keys(config.apps);

    for (let appName of apps) {
        require(FRAMEWORK_ROOT_PATH + '/bootstrap/app-config')(appName);

        if (command.options.baseUrl) {
            global.BASE_URL = command.options.baseUrl;
        }

        let { sassCompiler } = require(FRAMEWORK_ROOT_PATH + '/bootstrap');

        global.echo.sameLine = console.log;

        sassCompiler.setOptions({
            root: ROOT,
            srcDir: SRC_DIR,
            appDir: APP_DIR,
            appName: APP_NAME,
        });

        const buildApp = require('./../app-compiler');

        await buildApp();
        await sassCompiler.clear('ltr').compile('ltr');
        await sassCompiler.clear('rtl').compile('rtl');

        require('./../production/compiler')(appName, command);
    }
        
    echo(cli.yellowBright('Generating htacess file...'));
    createHtaccessFile(command, apps);

    echo(cli.greenBright('All applications have been built successfully.'));

    die();
}

module.exports = function (command) {
    startProducing(command);
}