const { fs } = require('./../../bootstrap/modules');
const Tag = require('./tag');
const HtmlCompiler = require('./html-compiler');
const childComponents = require('./../child-components');

/**
 * Child component is used to inject the content of the current component into its parent component instead of 
 * creating separated component  
 */
module.exports = class ChildComponent extends Tag {
    /**
     * {@inheritDoc}
     */
    build() {
        let componentFile = this.htmlFile;
        let htmlContent = fs.get(componentFile);
        let compiler = new HtmlCompiler(htmlContent);

        for (let child of compiler.root.childNodes) {
            this.htmlCompiler.extract(child, null, this.tagNamespace);
            this.htmlCompiler.observe(compiler.observableProperties);
        }

        let absoluteFilePath = normalizePath(this.htmlFile);

        // Map the current html file to its parents as when the current child component is saved
        // the parent should be re-parsed not this one
        childComponents.add(absoluteFilePath, normalizePath(this.htmlCompiler.originalCompiler.parser.filePath));

        this.originalElement.parentNode.removeChild(this.originalElement);
    }
}